@@ -1,6 +1,6 @@
 #!/bin/sh -e
 #
-# Copyright (C) 2010  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2009  Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -14,7 +14,7 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: sign.sh,v 1.4.6.3 2010/02/25 05:23:41 tbox Exp $
+# $Id: sign.sh,v 1.4.2.2 2010/01/07 17:17:19 each Exp $
 
 SYSTEMTESTTOP=../..
 . $SYSTEMTESTTOP/conf.sh
@@ -33,14 +33,3 @@ for domain in example example.com; do
 
 	$SIGNER -r $RANDFILE -o $zone $zonefile > /dev/null 2>&1
 done
-
-zone=example.
-infile=example.db.in
-zonefile=example.db
-
-keyname1=`$KEYGEN -r $RANDFILE -a RSASHA1 -b 768 -n zone $zone`
-keyname2=`$KEYGEN -r $RANDFILE -a RSASHA1 -b 1024 -f KSK -n zone $zone`
-
-cat $infile $keyname1.key $keyname2.key >$zonefile
-
-$SIGNER -r $RANDFILE -o $zone $zonefile > /dev/null
